# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2010-2012 Elico Corp. All Rights Reserved.
#    Author:            Eric CAUDAL <contact@elico-corp.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import logging
from translate import _

_logger = logging.getLogger(__name__)

#-------------------------------------------------------------
# CHINESE NUMBERS (actually not letters)
#-------------------------------------------------------------

to_9_cn_digits = ( '零', '一', '二', '三', '四', '五', '六', '七', '八', '九' )

to_1000_cn_digits  = ( '十', '百', '千' )

from_10000_cn_digits  = ( '', '万', '亿', '兆', '京' )



def _chinese_number(val):    
    if (val < 10):
        return to_9_cn_digits[val]
    
    power = 0
    powers = []
    cpt = 0
    cnumber = ''

    # Get the value of the coefficient for each power of ten
    while (10** power <= val):   
        my_value = (val % (10** (power+1)))/(10** power)      
        powers.insert(power, my_value)
        
        # Subtract out the current power's coefficient and increase the power
        val -= val % (10**(power+1))
        power += 1
        
        
    # Take the decomposition of the number for above and generate the Chinese equivalent
    while (cpt < power):
        # Reached the next four powers up level
        if ((cpt % 4) == 0):
            if (powers[cpt] != 0):                
                inzero = 0
                canaddzero = 1
                cnumber =  to_9_cn_digits[powers[cpt]] + from_10000_cn_digits[cpt/4] + cnumber
            else:
                # Check that something in the next three powers is non-zero before adding
                if (((cpt+3 < power) and powers[cpt+3] != 0) or ((cpt+2 < power) and powers[cpt+2] != 0) or ((cpt+1 < power) and powers[cpt+1] != 0)):
                    cnumber = from_10000_cn_digits[cpt/4] + cnumber
                    canaddzero = 0 # added
            
        # Add one, tens, hundreds, or thousands place for each level
        else:
            if (powers[cpt] != 0):
                inzero = 0
                canaddzero = 1
                if (power == 2 and cpt == 1 and powers[cpt] == 1):  # No 一 with 10 through 19
                    cnumber = to_1000_cn_digits[(cpt % 4)-1] + cnumber
                else:
                    cnumber = to_9_cn_digits[powers[cpt]] + to_1000_cn_digits[(cpt%4)-1] + cnumber
            else:
                # Only insert one 零 for all consecutive zeroes
                if (canaddzero == 1 and inzero == 0):
                    inzero = 1
                    cnumber = to_9_cn_digits[powers[cpt]] + cnumber
        cpt += 1
    
    return cnumber


def _chinese_cents(val):    
    if (int(val) < 1):
        return ''
    
    cnumber = ''
    cpt = 0
    
    while (cpt < len(val)):
        cnumber += to_9_cn_digits[int(val[cpt])]
        cpt += 1
    
    return cnumber



def amount_to_text(number, currency):
    number = '%.2f' % number
    units_name = currency
    list = str(number).split('.')
    start_word = _chinese_number(int(list[0]))
    end_word = _chinese_cents(str(list[1]))
    cents_number = int(list[1])

    print cents_number
    
    cents_name = (cents_number > 0) and '点' or ''
    final_result = start_word + cents_name + end_word
    return final_result


#-------------------------------------------------------------
# Generic functions
#-------------------------------------------------------------

_translate_funcs = {'cn_digits' : amount_to_text}
    
#TODO: we should use the country AND language (ex: septante VS soixante dix)
#TODO: we should use en by default, but the translation func is yet to be implemented
def amount_to_text(nbr, lang='cn_digits', currency='yuan'):
    """ Converts an integer to the chinese numbers (actually not letters. eg: 4 into number is 四 and into letter is 肆)
    
        Examples::  10  : 十   (10)
                    20  : 二十 (2x10)

                    99  : 九十九   (9x10 + 9)
                    100 : 一百     (1x100)
                    101 : 一百零一 (1x100 + 01)

                    110 : 一百一十    (1x100 + 1x10)
                    111 : 一百一十一  (1x100 + 1x10 + 1)
                    120 : 一百二十    (1x100 + 2x10)

                    1,000 : 一千        (1x1,000)
                    1,001 : 一千零一    (1x1,000 + 0 + 1)
                    1,010 : 一千零一十  (1x1,000 + 0 + 1x10)
                    1,100 : 一千一百    (1x1,000 + 1x100)

                    10,000      : 一万
                    1,000,000   : 一百万
                    100,000,000 : 一亿	
    """
    import openerp.loglevels as loglevels
#    if nbr > 99999999999999999999:
#        _logger.warning(_("Number too large '%d', can not translate it"))
#        return str(nbr)
    
    if not _translate_funcs.has_key(lang):
        _logger.warning(_("no translation function found for lang: '%s'"), lang)
        #TODO: (default should be en) same as above
        lang = 'cn_digits'
    return _translate_funcs[lang](abs(nbr), currency)


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
